package ai.carespace.unisys.stealth_id.stealth_id

import android.app.Activity
import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import androidx.appcompat.app.AppCompatActivity
import com.unisys.sisaas.android.authenticator.sisdk.models.CustomerInfo
import com.unisys.sisaas.android.authenticator.sisdk.models.FaceCaptureResults
import com.unisys.sisaas.android.authenticator.sisdk.models.aware.FaceCaptureLivenessResults
import com.unisys.sisaas.android.authenticator.sisdk.platform.external.enrollment.resultcallbacks.UserInfoCallback
import com.unisys.sisaas.android.authenticator.sisdk.platform.external.face.OnFaceCaptureListener
import com.unisys.sisaas.android.authenticator.sisdk.platform.external.face.OnLivenessFaceCaptureListener

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.unisys.sisaas.android.authenticator.sisdk.platform.external.setup.SIPlatform
import com.unisys.sisaas.android.authenticator.sisdk.platform.urls.SaaSMobileConfig
import com.unisys.sisaas.android.authenticator.sisdk.utils.PermissionUtils
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding

/** StealthIdPlugin */
class StealthIdPlugin() : FlutterPlugin, MethodCallHandler, ActivityAware, AppCompatActivity() {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var siPlatform : SIPlatform;
  private lateinit var context: Context
  private lateinit var activity: Activity
  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "stealth_id")
    channel.setMethodCallHandler(this)
    context = flutterPluginBinding.applicationContext
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

    when(call.method){
      "getPlatformVersion" -> {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")
      }
      "authenticateEnrolledUser" -> {
        val userId : String? = call.argument("userId")
        print("authenticateEnrolledUser -- Enter")
        // startCapture(userId , result)
        captureFace()
      }
      "enrollUser" -> {
        val userId : String? = call.argument("userId")
        print("enrollUser -- Enter")
        captureLiveFace(userId)

      }
      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  private fun createCustomerProfile(){
    try {
      siPlatform!!.customerProfileSetup.getCustomerProfileData("deviceId","carespacedoc", object:UserInfoCallback {
        override fun onSuccess(p0: CustomerInfo?) {
         Log.d("MainActivity","Created user profile info success")
        }

        override fun onError(p0: Throwable?) {
          Log.d("MainActivity","Created user profile info fail")
        }

      })
    } catch (e: Exception){
      throw  e
    }
  }

 private fun captureLiveFace(userId : String?) {
   try {
     siPlatform!!.faceModality.launchFaceLivenessDetector(this,userId, object : OnLivenessFaceCaptureListener {
       override fun faceLivenessSuccess(p0: FaceCaptureLivenessResults?) {
         TODO("Not yet implemented")
       }

       override fun faceLivenessTimeOut() {
         TODO("Not yet implemented")
       }

       override fun faceLivenessCaptureAbort() {
         TODO("Not yet implemented")
       }

       override fun faceLivenessErrorReporter(p0: Int, p1: String?) {
         TODO("Not yet implemented")
       }

       override fun onCancelClickException() {
         TODO("Not yet implemented")
       }

     })
   }catch(e : Exception) {
    print(e.message)
     throw e
   }
  }

  private fun captureFace(){
    siPlatform.faceModality.launchFaceDetector(this, object :OnFaceCaptureListener{
      override fun onFaceCaptured(p0: MutableList<FaceCaptureResults>?) {
        TODO("Not yet implemented")
      }

      override fun onError(p0: Throwable?) {
        TODO("Not yet implemented")
      }

      override fun onFaceCapturedWithError(p0: MutableList<FaceCaptureResults>?, p1: Throwable?) {
        TODO("Not yet implemented")
      }

      override fun onCancelClickException() {
        TODO("Not yet implemented")
      }

    })
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
   if(PermissionUtils.allPermissionsGranted(this)){
     siPlatform = SIPlatform(context, SaaSMobileConfig())
     // this.createCustomerProfile()
   }else{
     PermissionUtils.getRuntimePermissions(this)
   }

  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    TODO("Not yet implemented")
  }

  override fun onDetachedFromActivity() {
    TODO("Not yet implemented")
  }

}
