//
//  CarespaceService.swift
//  risecx_stealthId
//
//  Created by Crístofer Domingues on 14/04/21.
//

import Foundation
import SIPlatformSDK

public class CarespaceService {
    var viewController : FlutterViewController
    internal var baseURL = ""
    internal var apiHeader : [String : String]
    
    init(viewController: FlutterViewController, baseURL:String, apiHeader: [String : String]! = ["":""]) {
        self.viewController = viewController
        self.baseURL = baseURL
        self.apiHeader = apiHeader
    }
    
    public func takePicture(completionHandler:@escaping (Dictionary<String, Any>) -> Void) {
        print("takePicture -- Enter ")
         guard let facialHandler = SIPlatformSDKManager.shareManager.facialHandler() as? SIFacialHandler else { return }
        facialHandler.doTakePicture(viewController:self.viewController){(imageQualityResult) in
            guard let tempImageQualityResult = imageQualityResult else {
                completionHandler(["statusCode": "TAKE_PICTURE_FAIL", "success": false, "message":"Take Picture has failed",  "expectedResult": ""])
                return
            }
            if imageQualityResult?.detectionResult == SIFaceDetectionResult.Ok {
                
                completionHandler(["statusCode":"DETECTION_SUCCESS", "success": true, "message":"Image successfully captured!","expectedResult":tempImageQualityResult.capturedImage!])
            }
            
           else if imageQualityResult?.detectionResult == SIFaceDetectionResult.BadFace {
                completionHandler(["statusCode": "DETECTION_RESULT", "success": false, "message":"The captured Image has not a good quality",  "expectedResult": ""])
            }
            
           else if imageQualityResult?.detectionResult == SIFaceDetectionResult.MultiFace {
                completionHandler(["statusCode": "DETECTION_RESULT", "success": false, "message":"Multiple faces were detected",  "expectedResult": ""])
            }
           else {
                completionHandler(["statusCode": "DETECTION_RESULT", "success": false,"message":"Unknown face detection result, please try again.",  "expectedResult": ""])
           }
        }
    }
    
    
    public func takePictureFromAwareLiveness(userId : String, transactionId :String ,completionHandler:@escaping (Dictionary<String, Any>) -> Void){
        let defaultAwareTimeout : Int = 32
        let defaultAwareFaceModel : String = "Lightweight"
        guard let facialHandler = SIPlatformSDKManager.shareManager.facialHandler() as? SIFacialHandler else { return }
        facialHandler.doTakePictureFromAwareLiveness(viewController: self.viewController, captureTimeout: defaultAwareTimeout, userId: userId, workflow: "Charlie4", faceModel: defaultAwareFaceModel) { (imageQualityResult) in
            if let livenessResponseType =
            imageQualityResult?.livenessResponseType, livenessResponseType ==
                .Success, let livenessResponse = imageQualityResult?.livenessResponse{
                guard let video = livenessResponse["video"] as? [String : Any], let
                workflow_data = video["workflow_data"] as? [String : Any], let
                metaData = video["meta_data"] as? [String : Any] else {
                    completionHandler(["statusCode": "AWARE_LIVENESS_FAIL", "success": false, "message":"Invalid Json Result",  "expectedResult": ""])
                return
                }
                self.onSucessAwareLivenessSDKResult(workflow_data, metaData, transactionId, userId, imageQualityResult) { (onSuccessResult) in
                    completionHandler(onSuccessResult)
                }
            }
        }
    }
    
    private func onSucessAwareLivenessSDKResult( _ workflow_data : [String : Any] , _ metaData : [String : Any],_ transactionId :String, _ userId :String, _ imageQualityResult : SIImageQualityResult?, completionHandler:@escaping (Dictionary<String, Any>) -> Void){
        var workflow : String = ""

                if  let workflowTemp =  workflow_data["workflow"] as? String {

                    workflow = workflowTemp

                }

                var rotation : Int = 0

                if  let rotationTemp =  workflow_data["rotation"] as? Int {

                    rotation = rotationTemp
                }

                var imageFrames : [Dictionary<String, Any>] =  []

                if let frames = workflow_data["frames"] as? [Dictionary<String,Any>] {

                    for frame in frames {

                        let tags = frame["tags"] as? [String]

                        let timeStamp = frame["timestamp"] as? Double

                        let data = frame["data"] as? String

                        let imageFrame : Dictionary<String, Any> = ["tags":tags!, "timestamp":timeStamp!,"data":data!]

                        imageFrames.append(imageFrame)

                    }

                }

                

                var client_device_model : String = ""

                if  let client_device_modelTemp =  metaData["client_device_model"] as? String {

                    client_device_model = client_device_modelTemp

                }

                

                var client_version : String = ""

                if  let client_versionTemp =  metaData["client_version"] as? String {

                    client_version = client_versionTemp

                }
        
        self.awareLivenessCheckService(workflow, client_version, imageFrames, client_device_model, rotation, transactionId, userId, completionHandler: { (livenessServiceResult) in
            completionHandler(livenessServiceResult)
        })
    }
    
    
    private func awareLivenessCheckService(_ workflow : String, _ clientVersion : String, _ frames : [Dictionary<String,Any>],  _ clientDeviceBrand : String, _ rotation : Int ,_ transactionId : String, _ userId : String, completionHandler: @escaping (Dictionary<String, Any>) -> Void ){
        let url =  self.baseURL.appending(":30643/sisaas/awareliveliness/2.2.0/liveness")
                
                let firstData = frames.first?["data"] ?? "Worng Image String"

                let firstTags = frames.first?["tags"] ?? []

                let firstTimestamp = frames.first?["timestamp"] ?? 0

                

                let secondData = frames[1]["data"] ?? "Worng Image String"

                let secondTags = frames[1]["tags"] ?? []

                let secondTimestamp = frames[1]["timestamp"] ?? 0

                

                let thirdData =  frames[2]["data"] ?? "Worng Image String"

                let thirdTags = frames[2]["tags"] ?? []

                let thirdTimestamp = frames[2]["timestamp"] ?? 0

                

                let fourthData =  frames[3]["data"] ?? "Worng Image String"

                let fourthTags = frames[3]["tags"] ?? []

                let fourthTimestamp = frames[3]["timestamp"] ?? 0

                

                let fifthData = frames.last?["data"] ?? "Worng Image String"

                let fifthTags = frames.last?["tags"] ?? []

                let fifthTimestamp = frames.last?["timestamp"] ?? 0

                var meta : [String: Any] = [

                    "meta": [

                        "location" :"",

                        "retentionCode" : "",

                        "deviceID" : ""

                    ]

                ]

        
        let data : [String: Any] = [

            "data": [

                "transactionId" : transactionId,

                "livenessInfo"  : [

                    "video" : [

                        "workflow_data" : [

                            "workflow" : workflow,

                            "rotation" : rotation,

                            "frames" : [

                                [

                                    "data" : firstData,

                                    "tags" : firstTags,

                                    "timestamp" : firstTimestamp

                                ],

                                [

                                    "data" : secondData,

                                    "tags" : secondTags,

                                    "timestamp" : secondTimestamp

                                ],

                                [

                                    "data" : thirdData,

                                    "tags" : thirdTags,

                                    "timestamp" : thirdTimestamp

                                ],

                                [

                                    "data" : fourthData,

                                    "tags" : fourthTags,

                                    "timestamp" : fourthTimestamp

                                ],

                                [

                                    "data" : fifthData,

                                    "tags" : fifthTags,

                                    "timestamp" : fifthTimestamp

                                ]

                            ]

                        ],

                        "meta_data" : [

                            "client_device_brand" : clientDeviceBrand,

                            "username": userId,

                            "client_version": clientVersion

                        ],

                        "client_version" : clientVersion

                    ]

                ]

            ]

        ]
        meta.merge(dict: data)
        print(meta.jsonStringRepresentation!)
        self.webserviceCaller(url,requestHeader:self.apiHeader, httpBodyData: meta.toJson) { (apiResult) in
            completionHandler(apiResult)
        }
    }
    
   
    
    public func webserviceCaller(_ url: String, requestHeader: [String : String]! = nil, httpBodyData: Data! = nil, requestDictonary:  [String : String]! = nil, queryParams:[String : String]! = nil, method:SICommunicationSDK.HttpMethod! = .post,  completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        
        if JSONSerialization.isValidJSONObject(requestDictonary ?? ["":""]) {
            print("webserviceCaller -- Enter ")
            print("webserviceCaller -- URL "+url)
            print("webserviceCaller -- HEADER "+(requestHeader?.jsonStringRepresentation!)!)
            print("******************************************************************************** ")
            guard let communicationHandler = SIPlatformSDKManager.shareManager.communicationHandler() as? SICommunicationHandler else {
                return
            }
            communicationHandler.callWebService(httpRequestUrl: url,
                                                httpHeader: requestHeader,
                                                            httpQuery: nil,
                                                            bodyParam: requestDictonary,
                                                            httpMethod: method,
                                                            httpBody: httpBodyData) { (genericResult) in
            guard genericResult?.results?.response?.httpStatusCode != StatusCode.NoInternet.rawValue else {
                               debugPrint("Internet connection Issue")
                               completionHandler(["statusCode": "NO_INTERNET", "success": false, "message": "Internet Connection's failed", "expectedResult": ""])
                               return
                           }
                if let result = genericResult?.results ,let responseData = result.data {

                    do {
                        let convertedString = String(data: responseData, encoding: String.Encoding.utf8)

                        print(convertedString ?? "defaultvalue")
                        
                        let dict = try JSONSerialization.jsonObject(with:responseData, options: [])
                        
                        guard let responseDic = dict as? [String : Any] else {
                            completionHandler(["statusCode": "BAD_JSON", "success": false, "message": "Malformed Json", "expectedResult": ""])
                            return
                        }
                        
                        
                        if let metaDic = responseDic["meta"] as? [String : Any], let success = metaDic["success"] as? Bool {
                            if(success || responseDic["data"] != nil ){
                                print("returning the success result")
                                completionHandler((responseDic["data"] as? Dictionary<String,Any>) ?? [ "success": true ] )
                            }else{
                                print("returning the meta result")
                                completionHandler((responseDic["meta"] as? Dictionary<String,Any>)!)
                            }
                           
                        }else {
                           if let _ = responseDic["access_token"] as? String{
                                print("returning access_token success result")
                                completionHandler(responseDic)
                            }
                        }
                        
                      
                    } catch let jsonEroor {
                        print(jsonEroor)
                        completionHandler(["statusCode": "BAD_JSON", "success": false, "message": "Malformed Json", "expectedResult": ""])
                    }
                }
            }
            
        } else {
            debugPrint("InvaLID JSON")
            completionHandler(["statusCode": "BAD_REQUEST", "success": false, "message": "INAVLID JOSN", "expectedResult": ""])
        }
        print("webserviceCaller -- Exit ")
    }
    
    public func getHeaderCredential(completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        self.clientRegister { (registerResult) in
            guard let consumerKey = registerResult["consumerKey"], let consumerSecret = registerResult["consumerSecret"] else{
                completionHandler(registerResult)
                return
            }
            self.getAPIToken(consumerKey as? String,consumerSecret as? String){
                (getAPITokenResult) in
                guard let access_token = getAPITokenResult["access_token"] else{
                    completionHandler(getAPITokenResult)
                    return
                }
                completionHandler(["access_token":access_token])
            }
            
        }
    }
    private var basicAuthBase64String:String! {
        let username = "sid-subscriber"
        let password = "!m+Y4:ZDhwwc"
        let basicAuthString = String(format: "%@:%@", username, password)
        guard let basicAuthData = basicAuthString.data(using: .utf8) else {
            return ""
        }
        let basicAuthBase64String = basicAuthData.base64EncodedString()
        return basicAuthBase64String
    }
    
    private func clientRegister(completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        let urlString = self.baseURL.appending("/clientRegister")
        let registerHead : [String:String] = ["Authorization":"Basic \(basicAuthBase64String ?? "")", "Content-Type": "application/json"]
        let query : [String:String] = ["customerCode":"carespacedoc"]
        
        self.webserviceCaller(urlString, requestHeader: registerHead, queryParams: query, method: .get) { (Result) in
            if(Result.keys.contains("consumerKey") && Result.keys.contains("consumerSecret")){
                completionHandler(Result)
            }else{
                completionHandler(["statusCode": "CLIENT_REGISTER_FAIL", "success": false, "message":Result["message"] as? String ?? "Error When Trying to Register Client",  "expectedResult": ""])
            }
        }
    }
    
    private func getAPIToken(_ consumerKey:String?, _ consumerSecret:String?, completionHandler: @escaping (Dictionary<String, Any>) -> Void){
        let url = self.baseURL.appending("/getAPIToken")
        let registerHead : [String:String] = ["Authorization":"Basic \(basicAuthBase64String ?? "")", "Content-Type": "application/json", "consumerKey":consumerKey ?? "","consumerSecret": consumerSecret ?? ""]
        let query:[String : String] = ["grant_type" : "client_credentials"]
        self.webserviceCaller(url, requestHeader: registerHead,queryParams: query, method: .get) { (Result) in
            if(Result.keys.contains("access_token")){
                completionHandler(Result)
            }else{
                completionHandler(["statusCode": "GET_TOKEN_FAIL", "success": false, "message":Result["message"] as? String ?? "Error When Trying to get API Token",  "expectedResult": ""])
            }
        }
    }
    
}
